#############################################################################
# Algorithm to Chapter 5 - Computation by a loop structure (E 1.1)
#############################################################################
# M. Brandl 2008
#############################################################################

simulation<-function(anzahl,n,b,r)

{
  i<-1
  sammel<-0
  sammel2<-0

  for (i in 1:anzahl)
  {
	x1<-rnorm(n)
	Anorm<-function(b){1/(2*pnorm(b/2)-1)}

	psi<-function(x,b)
	{
	 Anorm(b)*pmin(b/2,pmax(x,-b/2))
	}

	#xo<-x1[order(psi(x1,b))] aber hier bereits monotone psi-Funktion
	xo<-x1[order(x1)]

	K<-0.5*n+1
	while (K>0.5*n)
	{
	K<-rbinom(1,n,r/sqrt(n))
   	}

	weg<-xo[1:K]
	hinzu<- -weg
	xoneu<-c(xo[(K+1):n],hinzu)

	start<-median(xoneu)
	onestep<-start + mean(psi(xoneu-start,b))
	twostep<-onestep + mean(psi(xoneu-onestep,b))
	threestep<-twostep + mean(psi(xoneu-twostep,b))

	empMSE<-(n*threestep^2)
	sammel[i]<-empMSE

	omega<-function(x,b)
	{
	max(psi(x,b))-min(psi(x,b))
	}

	asMSE_0<-mean((psi(xo,b))^2)+r^2*(omega(xo,b))^2
	sammel2[i]<-asMSE_0

	#cat(K, onestep, twostep, threestep, empMSE, mean(sammel), "\n")
	i<-i+1
	#print(c(K,empMSE))
   }
emMSE<-mean(sammel)
asMSE<-mean(sammel2)
return(c(emMSE,asMSE))
}

szenario<-function(anzahl1, anzahl2,n,b,r)
{
	ergebnis<-matrix(nrow=anzahl1,ncol=3)
	for (i in 1:anzahl1)
	{
	a<-simulation(anzahl2,n,b,r)
	ergebnis[i,]<-c(n,a[1],a[2])
	}
ergebnis
}


szenario<-function(start,ende, anzahl2,b,r)
{
	ergebnis<-matrix(nrow=ende-start+1,ncol=3)
	i<-start
	for (i in start:ende)
	{
	a<-simulation(anzahl2,i,b,r)
	ergebnis[i-start+1,]<-c(i,a[1],a[1]-a[2])
	i<-i+1
	}
ergebnis
}


#############################################################################
# Algorithm to Chapter 5 - Computation by matrix operation (E 1.2)
#############################################################################
# M. Brandl 2008
#############################################################################

simulation2<-function(anzahl,n,b,r)
{
Anorm<-function(b){1/(2*pnorm(b/2)-1)}
psi<-function(x,b){Anorm(b)*pmin(b/2,pmax(x,-b/2))}
X1<-rnorm(anzahl*n)
Xi<-matrix(X1,anzahl,n)
a1<-matrix(Xi,ncol=anzahl,byrow=T)
a2<-apply(a1,2,sort)
Xis<-matrix(a2,ncol=n,byrow=T)

Ka<-rbinom(anzahl,n,r/sqrt(n))
Kb<-n-Ka
K<-cbind(Ka,Kb)
K<-matrix(K,nrow=2,byrow=T)
K<-as.vector(K)
K1<-rep(rep(c(-1,1),anzahl),times=K)
K1mat<-matrix(K1,nrow=anzahl,ncol=n,byrow=T)

Kindex<-(apply(K1mat,1,sum)<0)

Xr<-Xis*K1mat
Xr<-Xr[!Kindex,]
anzahlneu<-sum(!Kindex)

start<-apply(Xr,1,median)
arg<-matrix(psi(Xr-start,b),anzahlneu,n)
onestep<-start + apply(arg,1,mean)
arg2<-matrix(psi(Xr-onestep,b),anzahlneu,n)
twostep<-onestep + apply(arg2,1,mean)
arg3<-matrix(psi(Xr-twostep,b),anzahlneu,n)
threestep<-twostep + apply(arg3,1,mean)

empMSE<-(n*threestep^2)
emMSE<-mean(empMSE)

arg4<-matrix(psi(Xis[!Kindex,],b),anzahlneu,n)
supXis<-apply(arg4,1,max)
infXis<-apply(arg4,1,min)
omega<-supXis-infXis

arg5<-apply(arg4^2,1,mean)
asMSE_0<-arg5+r^2*omega^2
asMSE<-mean(asMSE_0)

return(c(emMSE,asMSE))
}

szenario2<-function(start,ende, anzahl2,b,r)
{
	ergebnis<-matrix(nrow=ende-start+1,ncol=3)
	i<-start
	for (i in start:ende)
	{
	a<-simulation2(anzahl2,i,b,r)
	ergebnis[i-start+1,]<-c(i,a[1],a[1]-a[2])
	i<-i+1
	}
ergebnis
}

######################################################################
# Convex Contamination:
######################################################################
# M. Brandl 2008
######################################################################

simulation3<-function(anzahl,n,b,r)

{
  i<-1
  sammel<-0
  sammel2<-0

  for (i in 1:anzahl)
  {
	x1<-rnorm(n)
	Anorm<-function(b){1/(2*pnorm(b/2)-1)}

	psi<-function(x,b)
	{
	 Anorm(b)*pmin(b/2,pmax(x,-b/2))
	}

	K<-0.5*n+1
	while (sum(K)>0.5*n)
	{
	K<-rbinom(n,1,r/sqrt(n))
   	}

	xoneu<-(1-K)*x1 + K*100

	start<-median(xoneu)
	onestep<-start + mean(psi(xoneu-start,b))
	twostep<-onestep + mean(psi(xoneu-onestep,b))
	threestep<-twostep + mean(psi(xoneu-twostep,b))

	empMSE<-(n*threestep^2)
	sammel[i]<-empMSE

	omega<-function(x,b)
	{
	max(abs(psi(x,b)))
	}

	asMSE_0<-mean((psi(x1,b))^2)+r^2*(omega(x1,b))^2
	sammel2[i]<-asMSE_0

	#cat(K, onestep, twostep, threestep, empMSE, mean(sammel), "\n")
	i<-i+1
	#print(c(K,empMSE))
   }
emMSE<-mean(sammel)
asMSE<-mean(sammel2)
return(c(emMSE,asMSE))
}


szenario3<-function(start,ende, anzahl2,b,r)
{
	ergebnis<-matrix(nrow=ende-start+1,ncol=3)
	i<-start
	for (i in start:ende)
	{
	a<-simulation3(anzahl2,i,b,r)
	ergebnis[i-start+1,]<-c(i,a[1],a[1]-a[2])
	i<-i+1
	}
ergebnis
}